package nl.madline.opendp500;

import java.io.File;

/**
 * @author svmaris
 */
public class JDP500 {
	public static final String APPNAME = "Open DP-500";
	public static final String VERSION = "0.3";
	public static final String AUTHOR = "Stephan van Maris";

	public static final String defaultConfig = "open-dp500.conf";

	/**
	 * This class is only used to fire up the main JDPServer.class.
	 * It's a bit redundant right now, but I wanted to add some
	 * command-line parameter-checking in here. Right now, it takes
	 * one argument, namely the name and path of the configuration
	 * file. If none is given, the default will be assumed, as 
	 * specified above.
	 *
	 * @param args Commandline arguments
	 * 	
	 */
	public static void main(String[] args) {
		System.out.println(APPNAME + " " + VERSION + " by " + AUTHOR);

		if (args.length > 1) {
			printHelp();
		} else {
			// Read configfile
			String configFile = null;
			if (args.length == 0) {
				configFile = defaultConfig;
			} else {
				configFile = args[0];
			}

			File f = new File(configFile);

			if (f.exists()) {
				JDPConfig config = new JDPConfig(f);
				new JDPServer(config);

			} else {
				System.out.println("Can not open file " + configFile);
				System.exit(1);
			}
		}
	}

	/**
	 * Display some help on the commandline arguments.
	 */
	private static void printHelp() {
		System.out.println("");
		System.out.println(
			"Usage: java -jar open-dp500.jar <configuration file>");
		System.out.println("");
	}
}
