package nl.madline.opendp500;

import java.util.Vector;

/**
 * @author svmaris
 */
public class JDPPlaylist {
    private String name;
    private Vector contents;
    private String mediaType;

    /**
     * Default constructor.
     * 
	 * @param mediaType The media-type of this playlist (AUDIO/VIDEO/PICTURE)
	 * @param name The name of the playlist
	 */
	public JDPPlaylist(String mediaType, String name) {
        this.contents = new Vector();
        this.mediaType = mediaType;
        this.name = name;
    }

    /**
     * Add a file to the playlist
     * 
	 * @param filename The file to be added
	 */
	public void addFile(String filename) {
        JDPFile file = new JDPFile(filename,this.mediaType);

        if(!contents.contains(file)) {
            contents.add(file);
        }
        else {
            file =(JDPFile)contents.elementAt(contents.indexOf(file));
            file.setTimesPlayed(file.getTimesPlayed()+1);
        }
    }

    /**
     * Remove a file from the playlist
     * 
	 * @param filename The file to be removed
	 */
	public void removeFile(String filename) {
        contents.remove(filename);
    }

    /**
     * Get an array of String with all current files
     * 
	 * @return files An array containing all the files in the playlist
	 */
	public String [] getFiles() {
        String [] returnValue = new String[contents.size()];

        for (int i = 0; i < contents.size(); i++) {
            JDPFile f = (JDPFile)contents.elementAt(i);
            returnValue[i] = f.getFilename();
        }

        return returnValue;
    }

    /**
     * Returns true if the playlist is empty
     * 
	 * @return empty True if empty, false if not
	 */
	public boolean isEmpty() {
        return contents.isEmpty();
    }

    /**
     * Return the name of the playlist
     * 
	 * @return name The name of the playlist
	 */
	public String getName() {
        return this.name;
    }

    /**
     * Return the media type of the playlist
     * 
	 * @return mediaType The media type (AUDIO/VIDEO/PICTURE)
	 */
	public String getMediaType() {
        return mediaType;
    }

    /**
     * Remove a file from the playlist by it's number.
     * First entry is '1'.
     * 
	 * @param number The number of the file that needs to be removed
	 */
	public void removeFile(int number) {
        contents.removeElementAt(number-1);
    }

    /**
     * Return a vector with the contents of the playlist
     * 
	 * @return contents All files in the playlist as a Vector
	 */
	public Vector getContents() {
        return contents;
    }

    /**
     * Set the current mediaType to a given value
     * 
	 * @param currentMediaType The media type of the playlist
	 */
	public void setMediaType(String currentMediaType) {
        this.mediaType = currentMediaType;
    }
}
