/*
 * Created on Apr 21, 2003
  */
package nl.madline.opendp500.test;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * @author svmaris
 */
public class JDPClient
	extends JFrame
	implements ActionListener, ListSelectionListener {
	private int WIDTH = 500;
	private int HEIGHT = 500;
	private JPanel buttonPanel;
	private JButton listAudioButton;
	private JButton listVideoButton;
	private JButton listPictureButton;
	private JButton listRadioButton;
	private JList outputList;
	private String ip;
	private JScrollPane scroller;
	private Vector listButtons;
	private String mediaType;
	private String currentDir;

	/* for Web radio */
	/** client DP 500 parameters */
	private String PlayerID = "XXXXXXXXXXXX"; /* this works like this too */
	private String PlayerModel = "DP-500";
	private String PlayerBrand = "KiSS";
	private int KissLanguage = 1;
	private int PlayerFirmware = 265;
	private int PlayerVersion = 100;
	/* Web radio test parameters */
	private int KissPORT = 80;
	private String KissWebServer = "webradio.kiss-technology.com";

	/**
	 * @param ipOfServer
	 */
	public JDPClient(String ipOfServer) {
		super();
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Unable to find System Look and Feel");
		}

		ip = ipOfServer;
		listButtons = new Vector();
		currentDir = "";
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
		int xpos = (int) (d.width - WIDTH) / 2;
		int ypos = (int) (d.height - HEIGHT) / 2;

		setLocation(xpos, ypos);
		setSize(WIDTH, HEIGHT);

		listAudioButton = new JButton("Audio");
		listVideoButton = new JButton("Video");
		listPictureButton = new JButton("Pictures");
		listRadioButton = new JButton("WebRadio");

		listAudioButton.addActionListener(this);
		listVideoButton.addActionListener(this);
		listPictureButton.addActionListener(this);
		listRadioButton.addActionListener(this);

		buttonPanel = new JPanel(new GridLayout(1, 3));

		buttonPanel.add(listAudioButton);
		buttonPanel.add(listVideoButton);
		buttonPanel.add(listPictureButton);
		buttonPanel.add(listRadioButton);

		outputList = new JList();

		outputList.setCellRenderer(new JDPCellRenderer());
		outputList.addListSelectionListener(this);

		scroller = new JScrollPane(outputList);

		getContentPane().add("Center", scroller);
		getContentPane().add("South", buttonPanel);

		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		currentDir = "";
		if (e.getSource().equals(listAudioButton)) {
			mediaType = "AUDIO";
			doCommand("LIST AUDIO ||");
		} else if (e.getSource().equals(listVideoButton)) {
			mediaType = "VIDEO";
			doCommand("LIST VIDEO ||");
		} else if (e.getSource().equals(listPictureButton)) {
			mediaType = "PICTURE";
			doCommand("LIST PICTURE ||");
		} else if (e.getSource().equals(listRadioButton)) {
			mediaType = "WEBRADIO";
			doRadioCommand();
		}

	}

	/**
	 * Handle the input received from the server. Only directory-listings are
	 * supported at the moment.
	 * 
	 * @param string The command that was received from the server
	 */
	private void doCommand(String string) {
		listButtons.removeAllElements();
		outputList.removeAll();

		try {
			Socket s = new Socket(ip, 8000);

			PrintWriter out = new PrintWriter(s.getOutputStream(), true);
			BufferedReader in =
				new BufferedReader(new InputStreamReader(s.getInputStream()));
			out.println(string);
			String read = null;
			Vector buttons = new Vector();

			if (!currentDir.equals("")) {
				buttons.add("<< back");
				listButtons.add("");
			}

			while ((read = in.readLine()) != null) {
				listButtons.add(read);
				String[] parts = read.split("\\|");
				buttons.add(parts[0]);
			}

			outputList.setListData(buttons);
		} catch (UnknownHostException e) {
			String[] data = { "Unknown host: " + ip };
			outputList.setListData(data);
		} catch (IOException e) {
			String[] data = { "Could not connect to " + ip };
			outputList.setListData(data);
		}
	}

	private void doRadioCommand() {
		listButtons.removeAllElements();
		outputList.removeAll();
		Vector buttons = new Vector();
		JDPConnectWebRadio JDPconnectWebRadio;
		JDPconnectWebRadio =
			new JDPConnectWebRadio(
				this.PlayerID,
				this.KissWebServer,
				this.KissPORT,
				this.KissLanguage,
				this.PlayerFirmware,
				this.PlayerVersion,
				this.PlayerModel,
				this.PlayerBrand);
		JDPconnectWebRadio.ListRadio(this.listButtons, buttons);
		outputList.setListData(buttons);
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting()) {
			int selected = outputList.getSelectedIndex();
			System.out.println(selected);
			if (selected == 0 && !currentDir.equals("")) {
				String[] tmp = currentDir.split("/");
				String returnValue = "";
				for (int i = 0; i < tmp.length - 1; i++) {
					returnValue += tmp[i] + "/";
				}
				currentDir = returnValue;
				doCommand("LIST " + mediaType + " |" + returnValue + "|");
			} else if (selected >= 0) {
				String action = (String)listButtons.get(selected);

				String[] parts = action.split("\\|");
				String[] parts2 = parts[1].split("/");
				parts[1] = parts2[parts2.length - 1];
				System.out.println("action  " + action);
				if (currentDir.equals("")) {
					currentDir = parts[1];
				} else {
					if (!currentDir.endsWith("/")) {
						currentDir += "/";
					}

					currentDir += parts[1];
				}

				if (parts[2].equals("1")) {
					System.out.println(action);
					doCommand("LIST " + mediaType + " |" + currentDir + "|");
				} else {
					System.out.println("File selected, don't do anything");
				}
			}
		}
	}
}
