/*
 * Created on Apr 21, 2003
 */
package nl.madline.opendp500.web;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.Socket;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Properties;

import nl.madline.opendp500.JDP500;
import nl.madline.opendp500.JDPConfig;

/**
 * @author svmaris
 */
public class JDPWebClientHandler implements Runnable {
	private JDPWebServer server;
	private Socket client;
	private InputStream in;
	private PrintStream out;
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	 public JDPWebClientHandler(JDPWebServer server, Socket s) {
	 	this.server = server;
	 	client = s;
	 }
	 
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			String ip = client.getInetAddress().getHostAddress();
			boolean allowed = server.getDPServer().isAllowedToConnect(ip);
			
			in = new BufferedInputStream(client.getInputStream());
			out = new PrintStream(client.getOutputStream(),true);
			byte [] buf = new byte[4096];
			
			int n = in.read(buf);
		
			String s = bytesToString(buf, n);
		
			if(s.startsWith("POST")) {
				if(allowed) {

					JDPConfig c = server.getDPServer().getConfig();
				
					// Write config
					String [] lines = s.split("\n");
					String postData = lines[lines.length-1];
					
					server.getDPServer().debugMsg("POST data: " + postData);
				
					postData = URLDecoder.decode(postData,"UTF-8");
					server.getDPServer().debugMsg("POST data: " + postData);
				
					String [] vars = postData.split("&");
				
					for (int i = 0; i < vars.length; i++) {
						String [] varCombo = vars[i].split("=");
						String key = varCombo[0];
						String value = varCombo[1];
					
						c.setProperty(key,value);					
					}
				
					c.save();
					server.getDPServer().reloadConfig();
				}
			}			
			
			if(s.startsWith("GET") || s.startsWith("POST")) {
				server.getDPServer().debugMsg("GET detected");
				
				String [] parts = s.split(" ");
				String [] parts2 = parts[1].split("\\?");
				
				String file = parts2[0];
				
				if(!allowed && file.endsWith(".html")) {
					file = "/not_allowed.html";
				}
				
				if(file.equals("/")) {
					file += "index.html";
				}
				
				File f = new File(server.getDocumentRoot() + file);
				
				printHeader(f);				
				printFile(f);				
			}
			
			out.flush();
			
			out.close();
			client.close();
			
		} catch (IOException e) {
			server.getDPServer().debugMsg("An error occured in the Webserver while sending a file to the client");			
		}
	}
	
	/**
	 * Open a file and send it to the browser
	 * 
	 * @param f The file to be send to the prowser
	 */
	private void printFile(File f) {
		try {
			InputStream is = new FileInputStream(f);
			byte [] buf = new byte[1024];
			
			try {
				if(f.getName().endsWith(".html")) {
					String output = "";
					int n;
							
					while((n = is.read(buf)) > 0) {
						output += bytesToString(buf,n);
						buf = new byte[1024];
					}
					
					server.getDPServer().debugMsg("Length: "+output.length());
					output = replaceVars(output);
					
					out.print(output);
				}
				else {				
					while(is.read(buf) > 0) {
						out.write(buf);
						buf = new byte[1024];
					}
				}
			} catch (IOException e1) {
				server.getDPServer().debugMsg("An error occured in the WebServer while reading a requested file.");
			}
			
		} catch (FileNotFoundException e) {
			server.getDPServer().debugMsg("Can not find the requested file: " + f.getName());
		}
	}

	/**
	 * Replace the variables in the template
	 * 
	 * @param output The HTML-data to be parsed
	 * @return output The parsed data
	 */
	private String replaceVars(String output) {
		output = replace(output,"{appname}", JDP500.APPNAME);
		output = replace(output,"{version}", JDP500.VERSION);
		output = replace(output,"{author}", JDP500.AUTHOR);
		output = replace(output,"{nowPlaying}", server.getDPServer().getNowPlaying());
		output = replace(output,"{connectedPlayers}", server.getDPServer().getConnectedPlayers());
		output = replace(output,"{lastCommand}",server.getDPServer().getLastCommand());
				
		Properties p = server.getDPServer().getConfig().getProperties();
		Enumeration e = p.keys();
		
		while(e.hasMoreElements()) {
			String key = (String)e.nextElement();
			String value = p.getProperty(key);
			server.getDPServer().debugMsg(key+" = "+value);
			output = replace(output,"{"+key+"}",value);
		}
		
		return output;
	}

	/**
	 * Replaces a substring with another
	 * 
	 * @param str Source string
	 * @param pattern What to replace
	 * @param replace Replace with...
	 * @return result The resulting string
	 */
	private String replace(String str, String pattern, String replace) {
		int s = 0;
		int e = 0;
		StringBuffer result = new StringBuffer();
    
    	while ((e = str.indexOf(pattern, s)) >= 0) {
			result.append(str.substring(s, e));
			result.append(replace);
			s = e+pattern.length();
		}

		result.append(str.substring(s));
		return result.toString();
	}


	/**
	 * Converts an array of bytes to a String
	 * 
	 * @param buf Buffer to convert to a String
	 * @param length The number of bytes to convert
	 * @return s The resulting String
	 */
	private String bytesToString(byte[] buf, int length) {
		String s = new String("");
		
		for (int i = 0; i < length; i++) {				
			s += (char)buf[i];
		}
					
		return s;
	}

	/**
	 * Prints the default header to the browser
	 * 
	 * @param f The file that the browser requested.
	 */
	private void printHeader(File f) {
		String filename = f.getName();
		String contentType = null;
		
		if(filename.endsWith(".html")) {
			contentType = "text/html";
		}
		else if(filename.endsWith(".gif")) {
			contentType = "image/gif";
		}
		else if(filename.endsWith(".jpg")) {
			contentType = "image/jpeg";
		}
				
		String header = "HTTP/1.0 200 OK\r\n" +
						"Content-type: " + contentType + "\r\n\r\n";
		
		out.print(header);		
	}
}
